/*
    Cross-assembler's mainline for standalone use
    MWA 26 May 2023 ff
*/

#include "as.c"

enum {
    default_code_page_size = 1024,
};

static void help() {
    printf(
"Usage: xa [OPTION]... [PROG]\n"
"Notable uses and options:\n\n"
"  xa                       assemble and list from standard input\n"
"  xa SRC -o BIN            assemble SRC to executable BIN\n"
"  xa SRC -o BIN -b 1024    assemble SRC to BIN with bootloader offset 1024\n"
"  xa SRC -o BIN -p 16      assemble SRC to BIN with code page size 16\n"
"  xa -h                    display this help and exit\n"
"  xa -v                    show tokenization and string table in listing\n"
"\n"
"The default code page size is %i words, except for bootloaders, where\n"
"the default is 0 words. Use -p after -b to override bootloader page size.\n",
    default_code_page_size);
}

int main(int argc, char *argv[]) {

    int opt;
    int parm    ;
    int maybe_ok;
    char *endp;
    char *infile = NULL, *outfile = NULL;

    as_set_code_page_size(default_code_page_size);

    while ((opt = getopt(argc, argv, "b:ho:p:v")) != -1) {
        switch (opt) {
            case 'b':
                parm = strtol(optarg, &endp, 10);
                if (*endp)
                    goto bad;
                as_set_boot_code_offset(parm);
                as_set_code_page_size(0);
                break;
bad:
            case 'h':
            default:
                help();
                return 1;
            case 'o':
                outfile = optarg;
                break;
            case 'p':
                parm = strtol(optarg, &endp, 10);
                if (*endp)
                    goto bad;
                as_set_code_page_size(parm);
                break;
            case 'v':
                as_set_verbosity(1);
                break;
        }
    }

    if (argc == optind + 1)
        infile = argv[optind];

    setlocale(LC_NUMERIC, "");
    as_trace = !!getenv("TRACE");

    if (!infile)
        printf("\x1b" "c");

    if (infile)
        maybe_ok = as_assemble_from_filename(infile);
    else
        maybe_ok = as_assemble_from_stream(stdin, ":) ");

    if (outfile)
        as_write_executable_file(outfile);

    return maybe_ok;
}
