/*
    Cross-assembler's prerequisite functions and macros
    MWA 11 May 2023 ff
*/

#define ifnot(x) if (!(x))

int as_trace;                           // external linkage intentional

static void asfail(char *s)
{
    printf("as: %s\n", s);
    exit(1);
}

// Print a 36-bit number using _ as thousands and ~ as minus.
// The number format is only signed if the number is negative.
static void pprint(ulong num, int minus, int min_width)
{
    char buf[20], *b;
    long n = minus ? num - (1L << 36) : num;

    snprintf(buf, sizeof(buf), "%'*li", min_width, n);
    while (b = strchr(buf, ','))
        *b = '_';
    while (b = strchr(buf, '-'))
        *b = '~';
    printf("%s", buf);
}

/*
    Convert a 36-bit number into an IMM instruction that loads it.
    If it can't be done, returns 0 (which is not an IMM instruction).
    This function is used to decide between IMM and adding 0, as well
    as generating the IMM needed.

    Note that signage is irrelevant; these instructions don't know it.
    The destination register returns as 0 and can be filled in.
*/
static ulong make_imm(ulong n)
{
    enum { mask18 = 0777777 };

    int imm_l = n & mask18;
    int imm_h = n >> 18 & mask18;
    int imm_half;
    int opcode;

    // case 1: may represent a signed negative number within [-262143, -1]
    if (imm_h == mask18) {
        opcode = oc_IMN;
        imm_half = imm_l;
    }

    // case 2: [0, 262143]
    else if (imm_h == 0) {
        opcode = oc_IMP;
        imm_half = imm_l;
    }

    // case 3: positive multiples of 262144
    else if (imm_l == 0) {
        opcode = oc_IMH;
        imm_half = imm_h;
    }

    // case 4: positive multiples of 262145,
    // meaning the high and low 18 bits are identical
    else if (imm_h == imm_l) {
        opcode = oc_IMB;
        imm_half = imm_l;
    }

    // case 5: would require three instructions to generate
    else
        return 0;

    return (ulong) opcode << 27 | imm_half;
}
