/*
    c_pull.c

    This is the simulation's somewhat broken implementation of an
    array of eight pull resistors. Specifically:

    a) The array is polarized. 'com1' and 'com2' are in, and
       'r0' - 'r7' are out. I don't consider this to actually be broken, 
       but it's unusual enough to write up here.

    b) The simulation does not actually emulate resistors at all.

    c) The simulation does not support any weak logic levels that
       can be "pulled away from".

    d) The two common terminals are regarded as power connections,
       not inputs. Design rule checking in run-netlist.py ensures
       both are connected. Tweaks generated by the same script,
       quite a kludge actually, tell the simulator whether the
       outputs pull high or low. If pulling both ways, the simulator
       will (rightly) consider that a fatal error.
*/

typedef struct {
    Prefix pre;             // fields common to all components
    int state;              // hiz, lo, hi
    int updated;            // true if there has been a state update
    out_pin r[8];           // output
} c_pull;

// Pull resistors have common pins, not input pins. Tweaks were inserted
// during netlist generation to indicate which rails.
in_namer in_namer_pull[] = {
    NULL, NULL,
};

// Here is the name and byte offset of the struct's output pins.
out_namer out_namer_pull[] = {
    "r0", &((c_pull *) NULL)->r[0],
    "r1", &((c_pull *) NULL)->r[1],
    "r2", &((c_pull *) NULL)->r[2],
    "r3", &((c_pull *) NULL)->r[3],
    "r4", &((c_pull *) NULL)->r[4],
    "r5", &((c_pull *) NULL)->r[5],
    "r6", &((c_pull *) NULL)->r[6],
    "r7", &((c_pull *) NULL)->r[7],
    NULL, NULL,
};

// No initialization is needed.
void init_pull(void *v, int truth)
{
}

// There are no adjustments supported during simulation,
// but the netlist has tweaks to indicate what rail(s) are connected.
// Of course, more than one rail is very bad.
void tweak_pull(void *v, char *parm)
{
    c_pull *g = v;

    if (!strcmp(parm, "vdd"))  {
        if (g->state == lo)
            goto conflict;
        g->state = hi;
    }

    else if (!strcmp(parm, "gnd"))  {
        if (g->state == hi)
            goto conflict;
        g->state = lo;
    }

    else
        pf("e!.%s bad tweak %s", g->pre.name, parm);

    g->updated = 1;
    return;

conflict:
    pf("e!.%s is pulled to both power rails", g->pre.name);
}

// Here is the simulation.
void be_pull(void *v)
{
    c_pull *g = v;
    int i;

    ifnot (g->updated)
        return;
    g->updated = 0;

    for (i=0; i<8; i++)
        pin_output_after_delay(&g->r[i], g->state, 1);

    if (g->pre.trace)
        pf(".%s.r0-r8 going %s\n",
                g->pre.name, pin_val_text(g->state));
}
